import {
  createRouter,
  createWebHistory,
  createWebHashHistory,
} from "vue-router";
import NProgress from "@/utils/progress";
import { handleChangeHtmlFontSize } from "@/utils/common";
/** 自动导入全部静态路由，无需再手动引入！匹配 src/router/modules 目录（任何嵌套级别）中具有 .ts 扩展名的所有文件，除了 remaining.ts 文件
 * 如何匹配所有文件请看：https://github.com/mrmlnc/fast-glob#basic-syntax
 * 如何排除文件请看：https://cn.vitejs.dev/guide/features.html#negative-patterns
 */
const modules: Record<string, any> = import.meta.glob(
  ["./modules/**/*.ts", "!./modules/**/remaining.ts"],
  {
    eager: true,
  }
);
const routes = [];
Object.keys(modules).forEach((key) => {
  routes.push(
    ...(Array.isArray(modules[key].default) ? modules[key].default : [])
  );
});

console.log(routes);

const router = createRouter({
  history: createWebHistory('/'),
  routes,
  scrollBehavior(to, from, savedPosition) {
    return new Promise((resolve) => {
      if (savedPosition) {
        return savedPosition;
      } else {
        if (from.meta.saveScrollTop) {
          const top: number =
            document.documentElement.scrollTop || document.body.scrollTop;
          resolve({ left: 0, top });
        }
      }
    });
  },
});

router.beforeEach((to: ToRouteType, _from, next) => {
  NProgress.start();
  if (to.meta?.title) {
    document.title = to.meta?.title;
  }
  console.log("before to", to);
  next();
});

router.afterEach((to) => {
  // if (to.params?.mechanismId) {
  //   localStorage.setItem("mechanismId", to.params.mechanismId as string);
  // }
  console.log("after to", to);

  if (to.query?.caringMode == "open") {
    handleChangeHtmlFontSize();
  } else {
    document.documentElement.style.fontSize = "";
  }
  NProgress.done();
});

export default router;
