import { stringify, parse } from "qs";
import router from "@/router";
import { request } from "./http";
import { useRoute } from "vue-router";
const { VITE_APP_DOMAIN } = import.meta.env;
import axios from "axios";
import dayjs from "dayjs";

export function validateForm(formRef) {
  return new Promise((resolve, reject) => {
    formRef &&
      formRef.validate((valid) => {
        if (valid) {
          resolve(true);
        } else {
          resolve(false);
        }
      });
  });
}
export function limitConcurrency(promises, callback, concurrency = 2) {
  let runningPromises = 0;
  let completedPromises = 0;
  const n = promises.length;

  function runNextPromise() {
    while (
      runningPromises < concurrency &&
      completedPromises + runningPromises < n
    ) {
      const promise = promises[completedPromises + runningPromises];
      runningPromises++;

      promise()
        .then(() => {
          completedPromises++;
          runningPromises--;
          if (completedPromises === n) {
            callback();
          } else {
            runNextPromise();
          }
        })
        .catch((error) => {
          console.error(error);
          completedPromises++;
          runningPromises--;
          runNextPromise();
        });
    }
  }

  runNextPromise();
}
export function randomString(len = 32) {
  let $chars = "ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz0123456789";
  let maxPos = $chars.length;
  let pwd = "";
  for (let i = 0; i < len; i++) {
    pwd += $chars.charAt(Math.floor(Math.random() * maxPos));
  }
  return pwd;
}

export const debounce = (fn: Function, delay: number) => {
  let timeout: any = undefined;

  return (...args: any[]) => {
    if (timeout) {
      clearTimeout(timeout);
    }

    timeout = setTimeout(() => {
      fn(...args);
    }, delay);
  };
};
export function throttle(fn: Function, time: number) {
  //初始化时间
  let lastTime = 0;
  return function () {
    //记录当前函数触发时间
    const nowTime = Date.now();
    if (nowTime - lastTime > time) {
      fn.call(this);
      lastTime = nowTime;
    }
  };
}
export function deepClone<T>(obj: T): T {
  const isObject = (input: any): input is { [key: string]: any } => {
    return input !== null && typeof input === "object";
  };

  if (!isObject(obj)) {
    return obj;
  }

  if (Array.isArray(obj)) {
    return obj.map((item) => deepClone(item)) as unknown as T;
  }

  const clonedObj: { [key: string]: any } = {};

  for (const key in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, key)) {
      clonedObj[key] = deepClone(obj[key]);
    }
  }

  return clonedObj as T;
}

export function parseJSON(input: any, catchVal?: any): any {
  if (!input) return input;
  try {
    const parsed = JSON.parse(input);
    return parsed;
  } catch (error) {
    console.log(error);
    return catchVal != undefined ? catchVal : input;
  }
}
export function conversionAge(time) {
  if (!time) {
    return "-";
  }
  const timeStamp = new Date(time).getTime();
  const nowTimeStamp = new Date().getTime();
  const poopTimeStamp = nowTimeStamp - timeStamp;
  const age = parseInt(poopTimeStamp / 31536000000 + "");
  return age;
}

export const timeToYear = (time: number) => {
  const yearInMs = 365 * 24 * 60 * 60 * 1000; // 一年的毫秒数
  return Math.floor(time / yearInMs);
};

export const formatAge = (birthday) => {
  if (!birthday) return "";
  if (!isNaN(Number(birthday))) return birthday;
  let time = birthday.replace(/-/g, "/");
  const timeFormLen = time.includes("/") && time.split("/").length;
  if (timeFormLen < 3) {
    for (let i = 0; i < timeFormLen - 1; i++) {
      time += "/01";
    }
  }
  const offset = Date.now() - +new Date(time);
  return parseInt(String(offset / 86400000 / 365));
};

export const pyramidSort = (arr, sortFn) => {
  const n = arr.length;

  // 创建一个新数组，用于存储排序后的结果
  const result = new Array(n);
  // 先对原数组进行升序排序
  sortFn ? arr.sort(sortFn) : arr.sort((a, b) => a - b);

  // 使用两个指针分别指向金字塔的两侧
  let left = 0;
  let right = n - 1;

  // 从金字塔的底部向上构建排序结果
  for (let i = 0; i < n; i++) {
    if (i % 2 === 0) {
      // 偶数索引位置，从右侧取值
      result[right] = arr[i];
      right--;
    } else {
      // 奇数索引位置，从左侧取值
      result[left] = arr[i];
      left++;
    }
  }

  return result;
};

export const parseJsonData = (data, defaultValue = "") => {
  try {
    return data
      ? typeof data == "string"
        ? JSON.parse(data)
        : data
      : defaultValue;
  } catch (e) {
    console.log(e);
    return defaultValue;
  }
};

export const finishLoad = () => {
  const div = document.createElement("div");
  div.id = "page-loaded";
  div.style.display = "none";
  document.body.appendChild(div);
};
export const deduplicateArrayByKeys = (arr, keys) => {
  const seen = new Set(); // 用于存储已经出现过的键值组合
  return arr.filter((item) => {
    const itemKeys = keys.map((key) => item[key]).join("-");
    if (!seen.has(itemKeys)) {
      seen.add(itemKeys);
      return true; // 如果键值组合不重复，则保留该项
    }
    return false; // 如果键值组合重复，则过滤掉该项
  });
  // 暂时屏蔽去重
  // return arr
};

export const inputBlur = () => {
  console.log("inputBlur");
  // setRootScrollTop(offset !== undefined ? offset : getRootScrollTop())
  // setRootScrollTop(0)
  window.scrollTo(0, 0);
};
export function wxPlusReady() {
  axios
    .get(`/jssdk`, {
      params: {
        url: encodeURIComponent(getCurrURL()),
        type: 2,
      },
      hideLoading: true,
    })
    .then((res) => {
      const { jsApiList, openTagList } = res.data.config;
      res.data.config.jsApiList = [...jsApiList];
      res.data.config.openTagList = [
        ...new Set([...openTagList, "wx-open-subscribe"]),
      ];
      window["wx"].config(res.data.config);

      window["wx"].ready(() => {
        window["wx"].hideAllNonBaseMenuItem();
      });

      window["wx"].error((res: any) => {
        console.log(res);
      });
    });
}

export function msToDate(ms: any, type = "dateTime") {
  const dateTime = new Date(ms);
  const year = dateTime.getFullYear();
  const month = dateTime.getMonth();
  const date = dateTime.getDate();
  const hour = dateTime.getHours();
  const minute = dateTime.getMinutes();
  const second = dateTime.getSeconds();

  const resultDateTime =
    year +
    "-" +
    (month + 1 >= 10 ? month + 1 : "0" + (month + 1)) +
    "-" +
    (date + 1 <= 10 ? "0" + date : date) +
    " " +
    (hour + 1 <= 10 ? "0" + hour : hour) +
    ":" +
    (minute + 1 <= 10 ? "0" + minute : minute) +
    ":" +
    (second + 1 <= 10 ? "0" + second : second);
  const resultDate =
    year +
    "-" +
    (month + 1 >= 10 ? month + 1 : "0" + (month + 1)) +
    "-" +
    (date + 1 <= 10 ? "0" + date : date);
  switch (type) {
    case "year":
      return year.toString();
    case "year-month":
      return `${year}-${month + 1 >= 10 ? month + 1 : "0" + (month + 1)}`;
    case "dateTime":
      return resultDateTime;
    default:
      return resultDate;
  }
}

export function getObjectURL(file: File) {
  let url = null;
  if (window["createObjectURL"]) {
    // basic
    url = window["createObjectURL"](file);
  } else if (window.webkitURL !== undefined) {
    // webkit or chrome
    try {
      url = window.webkitURL.createObjectURL(file);
    } catch (error) {
      //
    }
  } else if (window.URL !== undefined) {
    // mozilla(firefox)
    try {
      url = window.URL.createObjectURL(file);
    } catch (error) {
      //
    }
  }
  return url;
}

export function downloadReportRename(downloadUrl, filename) {
  return new Promise((resolve, reject) => {
    const xhr = new XMLHttpRequest();
    xhr.open("GET", downloadUrl, true);
    xhr.responseType = "blob";
    xhr.onload = () => {
      if (xhr.status === 200) {
        const link = document.createElement("a");
        const body = document.body;
        const blob = xhr.response;
        link.href = window.URL.createObjectURL(blob);
        link.download = filename;

        // hide the link
        link.style.display = "none";
        body.appendChild(link);

        link.click();
        body.removeChild(link);

        window.URL.revokeObjectURL(link.href);
        resolve(link.href);
      } else {
        reject(new Error(`Request failed with status ${xhr.status}`));
      }
    };
    xhr.onerror = () => {
      reject(new Error("Request failed"));
    };
    xhr.send();
  });
}
export function isWx() {
  const ua = navigator.userAgent.toLowerCase();
  if (ua && ua.includes("micromessenger")) {
    return new Promise((resolve) => {
      window["wx"].miniProgram.getEnv(function (res: any) {
        if (res.miniprogram) {
          resolve("mini-wx");
        } else {
          resolve("wx");
        }
      });
    });
  } else {
    console.log("不在微信里");
    return new Promise((resolve) => {
      resolve("no-wx");
    });
  }
}

export const isMiniProgram = async () => {
  const type = await isWx();
  return type === "mini-wx";
};

export const miniProgramBack = async (cb?: Function, opt?: any) => {
  const flag = await isMiniProgram();
  if (flag && window["wx"]?.miniProgram?.[opt?.method || "navigateBack"]) {
    window["wx"]?.miniProgram?.[opt?.method || "navigateBack"](opt?.option);
  } else {
    cb && cb();
  }
};
export const miniProgramDownloadFile = async (cb?: Function, opt?: any) => {
  const flag = await isMiniProgram();
  if (flag && window["wx"]?.miniProgram?.[opt?.method || "navigateTo"]) {
    window["wx"]?.miniProgram?.[opt?.method || "navigateTo"]({
      url: `/pages/download/index?url=${opt.url}`,
    });
  } else {
    cb && cb();
  }
};

export function isIOS() {
  return /iPhone|iPad|iPod/i.test(navigator.userAgent);
}

export const handleChangeHtmlFontSize = () => {
  // document.querySelector("html").style.fontSize = `${18}px`;
  // 基准大小
  const baseSize = 20;
  // 设置 rem 函数
  function refreshRem() {
    // 当前页面宽度相对于 750 宽的缩放比例，可根据自己需要修改。
    const scale = document.documentElement.clientWidth / 750;
    // 设置页面根节点字体大小
    document.documentElement.style.fontSize = baseSize + "px";
  }
  // 初始化
  refreshRem();
  let tid;
  window.addEventListener(
    "resize",
    function () {
      clearTimeout(tid);
      tid = setTimeout(refreshRem, 300);
    },
    false
  );
  window.addEventListener(
    "pageshow",
    function (e) {
      if (e.persisted) {
        // 页面从浏览器的缓存中读取
        clearTimeout(tid);
        tid = setTimeout(refreshRem, 300);
      }
    },
    false
  );
};
